# -*- coding: utf-8 -*-
#
# This file is part of DT assessment framework.
#
# This code is developed to support the framework to estimate earthquake induced downtime 
# and recovery trajectory of buildings 
# proposed by C. Molina Hutt, T. Vahanvaty, P. Kourehpaz. 
#
# The proposed framework can be found in the manuscript entitled 
# "An analytical framework to assess earthquake induceddowntime and model recovery of buildings"
# The manuscript will be submitted to Earthquake Spectra for publication.
# 
# This code will be publicly available on GitHub after the manuscript is published, 
# and it should not be distributed until then.
#
# Contributor(s):
# Pouria Kourehpaz


"""
This module calculates impeding factor delays

"""

def IF_calc(n_realizations, total_cost, coeff_financing, RCmax_RS, N_DMG_RC5, N_DMG_RC3_RS, DMG_input, DL_summary_input):
    import numpy as np
    import pandas as pd
    #import timeit
    #start = timeit.default_timer()
    
    #n_realizations = 2000
    DMG = pd.read_csv(DMG_input, low_memory=False, header=None)
    PG = DMG.loc[1, 1:] #performance groups indicating the location of fragility
    
    
    FG = DMG.loc[0, 1:] #fragility groups
    #FG_unique = FG.drop_duplicates()
    n_PG = len(FG)
    DL_summary=pd.read_csv(DL_summary_input)
    
    
    cases_collapse = DL_summary["collapses/collapsed"]
    indx_collapse = cases_collapse[cases_collapse==1].index
    
    cases_irreparable = DL_summary["reconstruction/irreparable"]
    indx_irreparable = cases_irreparable[cases_irreparable==1].index
    
    indx_all = DL_summary["#Num"]
    indx_repairable =  indx_all.drop(indx_collapse.union(indx_irreparable))
    
   
    
    story_FG = []
    for i in range(n_PG):
        story_FG.append((PG[i+1][len(PG[i+1])-4:len(PG[i+1])-1]))
    
    story = list(set(story_FG))
    story.sort()
    
    #%% repairable buildings -- impeding factors
    #inspection
    IF_inspection = np.zeros(len(indx_repairable))
    
    for i in range (len(RCmax_RS)):
        if (max(RCmax_RS[i,:])>=1):
           IF_inspection[i]= np.random.lognormal(np.log(5),0.54)
    IF_inspection_mat2 = np.append(IF_inspection, np.zeros(len(indx_irreparable)+len(indx_collapse)))
    
    #engineering
    n1=np.zeros(len(indx_repairable))
    n2=np.zeros(len(indx_repairable))
    for i in range (len(indx_repairable)):
        x=0
        for j in range(len(story)):
            if ((RCmax_RS[i,x])==1):
                n1[i]=n1[i]+1
            elif ((RCmax_RS[i,x])>1):
                n2[i]=n2[i]+1
            x = x+7
    IF_eng = (n1*np.random.lognormal(np.log(42),0.4,len(indx_repairable)) + n2*np.random.lognormal(np.log(84),0.4,len(indx_repairable)))/len(story)
    IF_eng_mat2 = np.append(IF_eng, np.zeros(len(indx_irreparable)+len(indx_collapse)))
    
    #permitting
    IF_permit = (n1*np.random.lognormal(np.log(7),0.86,len(indx_repairable)) + n2*np.random.lognormal(np.log(56),0.32,len(indx_repairable)))/len(story)
    IF_permit_mat2 = np.append(IF_permit, np.zeros(len(indx_irreparable)+len(indx_collapse)))
    
    #financing
    #total_cost = 47564000 #input
    repair_cost_all = DL_summary[DL_summary.iloc[:,3]==0]
    repair_cost = repair_cost_all.iloc[:,4]
    #repair_cost = pd.read_csv(r'DV_rec_cost_agg.csv',low_memory=False, header=None).loc[1:,1:].to_numpy(dtype='float')
    ELR = (repair_cost/total_cost).to_numpy()
    deductible = 0.1
    
    #coeff_financing = [0.16, 0.72, 0.12]
    IF_finance = np.zeros(len(indx_repairable))
    rand_num = np.random.rand(len(indx_repairable))
    for i in range(len(indx_repairable)):
         #IF_Insur calculation
        if rand_num[i] <=coeff_financing[0]:
            if (ELR[i]>0.05) and (ELR[i]<=deductible):
                IF_finance[i] = 0.5*np.random.lognormal(np.log(105),0.68)
            elif (ELR[i]>deductible):
                IF_finance[i] = max(0.5*np.random.lognormal(np.log(105),0.68) , np.random.lognormal(np.log(42),1.11))
         #IF_PrivLoan calculation
        elif coeff_financing[0]< rand_num[i] <=coeff_financing[1]+coeff_financing[0]:
            if (ELR[i]>0.05) and (ELR[i]<=0.1):
                IF_finance[i] = 0.5*np.random.lognormal(np.log(105),0.68)
            elif (ELR[i]>0.1):
                IF_finance[i] = np.random.lognormal(np.log(105),0.68)
        #IF_PubLoan calculation    
        else: 
            if (ELR[i]>0.1):
                IF_finance[i] = np.random.lognormal(np.log(336),0.57)
            else:
                IF_finance[i] = 0.5*np.random.lognormal(np.log(336),0.57)
    
    IF_finance_mat2 = np.append(IF_finance, np.zeros(len(indx_irreparable)+len(indx_collapse)))
    
    #contractor mobilization

    IF_cm_rs1 = (n1*np.random.lognormal(np.log(100.8),0.39,len(indx_repairable)) + n2*np.random.lognormal(np.log(150.5),0.33,len(indx_repairable)))/len(story)
    IF_cm_rs1_mat2 = np.append(IF_cm_rs1, np.zeros(len(indx_irreparable)+len(indx_collapse)))
    
    #IF_cm_rs2_mat = np.zeros([len(RCmax_RS), len(story)])
    n1=np.zeros(len(indx_repairable))
    n2=np.zeros(len(indx_repairable))
    for i in range (len(RCmax_RS)):
        x=1
        for j in range(len(story)):
            if ((RCmax_RS[i,x])==1):
                n1[i]=n1[i]+1
            elif ((RCmax_RS[i,x])>1):
                n2[i]=n2[i]+1
            x = x+7
    IF_cm_rs2 = (n1*np.random.lognormal(np.log(50.4),0.35,len(indx_repairable)) + n2*np.random.lognormal(np.log(122.5),0.43,len(indx_repairable)))/len(story)
    IF_cm_rs2_mat2 = np.append(IF_cm_rs2, np.zeros(len(indx_irreparable)+len(indx_collapse)))
    
    n1=np.zeros(len(indx_repairable))
    n2=np.zeros(len(indx_repairable))
    for i in range (len(RCmax_RS)):
        x=2
        for j in range(len(story)):
            if ((RCmax_RS[i,x])==1):
                n1[i]=n1[i]+1
            elif ((RCmax_RS[i,x])>1):
                n2[i]=n2[i]+1
            x = x+7
    IF_cm_rs3 = (n1*np.random.lognormal(np.log(91.7),0.53,len(indx_repairable)) + n2*np.random.lognormal(np.log(145.6),0.81,len(indx_repairable)))/len(story)  
    IF_cm_rs3_mat2 = np.append(IF_cm_rs3, np.zeros(len(indx_irreparable)+len(indx_collapse)))
        
    n1=np.zeros(len(indx_repairable))
    n2=np.zeros(len(indx_repairable))
    for i in range (len(RCmax_RS)):
        x=3
        for j in range(len(story)):
            if ((RCmax_RS[i,x])==1):
                n1[i]=n1[i]+1
            elif ((RCmax_RS[i,x])>1):
                n2[i]=n2[i]+1
            x = x+7
    IF_cm_rs4 = (n1*np.random.lognormal(np.log(85.4),0.47,len(indx_repairable)) + n2*np.random.lognormal(np.log(135.1),0.86,len(indx_repairable)))/len(story) 
    IF_cm_rs4_mat2 = np.append(IF_cm_rs4, np.zeros(len(indx_irreparable)+len(indx_collapse)))
    
    n1=np.zeros(len(indx_repairable))
    n2=np.zeros(len(indx_repairable))
    for i in range (len(RCmax_RS)):
        x=4
        for j in range(len(story)):
            if ((RCmax_RS[i,x])==1):
                n1[i]=n1[i]+1
            elif ((RCmax_RS[i,x])>1):
                n2[i]=n2[i]+1
            x = x+7
    IF_cm_rs5 = (n1*np.random.lognormal(np.log(60.2),0.43,len(indx_repairable)) + n2*np.random.lognormal(np.log(79.1),0.3,len(indx_repairable)))/len(story)
    IF_cm_rs5_mat2 = np.append(IF_cm_rs5, np.zeros(len(indx_irreparable)+len(indx_collapse)))
    
    n1=np.zeros(len(indx_repairable))
    n2=np.zeros(len(indx_repairable))
    for i in range (len(RCmax_RS)):
        x=5
        for j in range(len(story)):
            if ((RCmax_RS[i,x])==1):
                n1[i]=n1[i]+1
            elif ((RCmax_RS[i,x])>1):
                n2[i]=n2[i]+1
            x = x+7
    IF_cm_rs6 = (n1*np.random.lognormal(np.log(130.9),0.59,len(indx_repairable)) + n2*np.random.lognormal(np.log(192.5),0.49,len(indx_repairable)))
    IF_cm_rs6_mat2 = np.append(IF_cm_rs6, np.zeros(len(indx_irreparable)+len(indx_collapse)))
    
    n1=np.zeros(len(indx_repairable))
    n2=np.zeros(len(indx_repairable))
    for i in range (len(RCmax_RS)):
        x=6
        for j in range(len(story)):
            if ((RCmax_RS[i,x])==1):
                n1[i]=n1[i]+1
            elif ((RCmax_RS[i,x])>1):
                n2[i]=n2[i]+1
            x = x+7
    IF_cm_rs7 = (n1*np.random.lognormal(np.log(57.4),0.39,len(indx_repairable)) + n2*np.random.lognormal(np.log(122.1),0.94,len(indx_repairable)))/len(story)
    IF_cm_rs7_mat2 = np.append(IF_cm_rs7, np.zeros(len(indx_irreparable)+len(indx_collapse)))
    
    # Stabilization
    IF_stab_RC5 = np.zeros((len(RCmax_RS))) #structrual repairs
    IF_stab_mob = 0
    N_DMG_RC5_tot=sum(N_DMG_RC5.transpose())
    for i in range(len(RCmax_RS)):
        if N_DMG_RC5_tot[i]>0 and N_DMG_RC5_tot[i] <=3:
            IF_stab_RC5[i] = IF_stab_mob + N_DMG_RC5_tot[i]*np.random.lognormal(np.log(6),0.4)
        elif (N_DMG_RC5_tot[i] >3) and (N_DMG_RC5_tot[i] <7):
           IF_stab_RC5[i] = IF_stab_mob + N_DMG_RC5_tot[i]*np.random.lognormal(np.log(-0.5*N_DMG_RC5_tot[i]+7.5),0.4)
        elif N_DMG_RC5_tot[i] >=7:
           IF_stab_RC5[i] = IF_stab_mob + (N_DMG_RC5_tot[i]*np.random.lognormal(np.log(4),0.4))/2
           
    IF_stab_RC3 = np.zeros((len(RCmax_RS))) #curtain walls
    N_DMG_RC3_RS_mat = np.squeeze(N_DMG_RC3_RS)[:,np.arange(2,len(np.squeeze(N_DMG_RC3_RS).transpose()),7)]
    N_DMG_RC3_RS3 = sum(N_DMG_RC3_RS_mat.transpose())
    
    for i in range(len(RCmax_RS)):
        if N_DMG_RC3_RS3[i]>0 and N_DMG_RC3_RS3[i] <=20:
            IF_stab_RC3[i] = IF_stab_mob + N_DMG_RC3_RS3[i]*np.random.lognormal(np.log(0.1429),0.4)
        elif N_DMG_RC3_RS3[i]>20 and N_DMG_RC3_RS3[i] <100:
            IF_stab_RC3[i] = IF_stab_mob + N_DMG_RC3_RS3[i]*np.random.lognormal(np.log(-0.0008375*N_DMG_RC3_RS3[i] + 0.15975),0.4)
        elif N_DMG_RC3_RS3[i]>= 100:
            IF_stab_RC3[i] = IF_stab_mob + N_DMG_RC3_RS3[i]*np.random.lognormal(np.log(0.076),0.4)/2
    
    IF_stab = np.maximum(IF_stab_RC3, IF_stab_RC5) #total stabilization time
    IF_stab_mat2 = np.append(IF_stab, np.zeros(len(indx_irreparable)+len(indx_collapse)))
            
        
    #%% irreparable buildings -- impeding factors
    
    IF_reconst_eng = np.random.lognormal(np.log(350),0.32,len(indx_irreparable)+len(indx_collapse))
    IF_reconst_insur = np.random.lognormal(np.log(224),1.0,len(indx_irreparable)+len(indx_collapse))
    IF_reconst_demol = np.random.lognormal(np.log(445),0.57,len(indx_irreparable)+len(indx_collapse))
    
    IF_reconst = np.maximum(IF_reconst_eng, IF_reconst_insur, IF_reconst_demol)
    IF_reconst_mat2 = np.append(np.zeros(len(indx_repairable)),IF_reconst)

    IF_matrix = [IF_inspection_mat2.astype(object), IF_eng_mat2.astype(object), IF_permit_mat2.astype(object), IF_finance_mat2.astype(object), IF_cm_rs1_mat2.astype(object), 
                 IF_cm_rs2_mat2.astype(object), IF_cm_rs3_mat2.astype(object), IF_cm_rs4_mat2.astype(object), IF_cm_rs5_mat2.astype(object), 
                 IF_cm_rs6_mat2.astype(object), IF_cm_rs7_mat2.astype(object), IF_stab_mat2.astype(object), IF_reconst_mat2.astype(object)]
    
    (pd.DataFrame(IF_matrix)).T.to_csv('IF_delays.csv', header=['IF_inspection','IF_eng','IF_permit','IF_finance',
                                                                'IF_cm_RS1','IF_cm_RS2','IF_cm_RS3','IF_cm_RS4',
                                                                'IF_cm_RS5','IF_cm_RS6','IF_cm_RS7','IF_stab','IF_reconstruct'], index_label='#Num')

    print('Impeding factor calculation is completed')
    output = [IF_inspection, IF_eng, IF_permit, IF_finance, IF_cm_rs1, IF_cm_rs2, IF_cm_rs3, IF_cm_rs4, IF_cm_rs5, IF_cm_rs6, IF_cm_rs7, IF_stab, IF_reconst]
    #stop = timeit.default_timer()
    #print('Time: ', stop - start)
    return output
    
    
