# -*- coding: utf-8 -*-
#
# This file is part of DT assessment framework.
#
# This code is developed to support the framework to estimate earthquake induced downtime 
# and recovery trajectory of buildings 
# proposed by C. Molina Hutt, T. Vahanvaty, P. Kourehpaz. 
#
# The proposed framework can be found in the manuscript entitled 
# "An analytical framework to assess earthquake induceddowntime and model recovery of buildings"
# The manuscript will be submitted to Earthquake Spectra for publication.
# 
# This code will be publicly available on GitHub after the manuscript is published, 
# and it should not be distributed until then.
#
# Contributor(s):
# Pouria Kourehpaz

"""
Define input parameters and files and execute this script to perform DT assessment

"""
import timeit
start = timeit.default_timer()
import DT_calculation

n_realizations = 2000
n_elev = 2
rep_phases = [3,3,3,3,4]    #last element should indicate #of basement levels, if none, insert 0.
bld_area_ft2 = 160000
total_cost = 47564000
coeff_financing = [0.16, 0.72, 0.12]    #entry 1,2,3: contribution from (1) insurance, (2) private loan, (3) public loan
Qt_facade = 12*2010/30 # input total facade quantity FEMA equivalent
reconst_time = 14 #reconstruction time per story in days

RCtable_input = 'Repair_Class_Table.xlsx'
DMG_input = 'DMG.csv'
DL_summary_input = 'DL_summary.csv'
DV_rec_time_input = 'DV_rec_time.csv'


parameters=[n_realizations, n_elev, rep_phases, bld_area_ft2, total_cost, coeff_financing, Qt_facade, reconst_time]
files = [RCtable_input, DMG_input, DL_summary_input, DV_rec_time_input]
DT_calculation.run_analysis(parameters, files)

stop = timeit.default_timer()
print('Time: ', stop - start)