function RUN_ANALYSIS(n_Floors,n_Floors_AG,n_real,Replacement_Cost)
% Downtime_Pelicun(16,12,2000,47564000)

% Please refer to README and input generation data at the end of this script

% Variable Definitions
% n_PG == total number of performance groups in model = 715
% n_Floors == total number of floors (above grade and below grade) = 16
% n_Floors_AG == number of floors above ground (i.e. excluding basement levels) = 12
% n_real == total number of realizations = 2000
% Replacement_Cost == Total Building Replacement Cost = $47564000

                  
%Data Imports                   ( Read data carefully and make neccessary modifications to extents, file names)

% Folder location
Folder_Path = 'C:\Users\taiks\Desktop\Thesis\Working\S12-2014-PG2_475 yr\';

% Run the script that prepares repair class matrix and prepares other input
% files comaptible with this MATLAB script
PelicunData2MATLAB(Folder_Path,n_real)

%   -modify extents1,extents3,to n_real 
%Data Imports  
filename1=append(Folder_Path,'PGTimes_All.xlsx');
filename2=append(Folder_Path,'RS_Story_Identifier.xlsx');
filename3=append(Folder_Path,'Repair_Class_All.xlsx');
filename4=append(Folder_Path,'AllReal_RDCases.xlsx');
filename5=append(Folder_Path,'AllReal_CollapseCases.xlsx');
filename6=append(Folder_Path,'AllReal_TotalLoss.xlsx');

%1) Repair Time Data 
Repair_Time_All=xlsread(filename1,1);

%2) Repair Class Data 
RS_Story_Identifier=xlsread(filename2,1);

%3) Repair Class Matrix 
Repair_Class_All=xlsread(filename3,1);

%4) Number of damaged components for each PG in each floor for each recovery state
filename10=append(Folder_Path,'Number_DC_RC2.xlsx');
filename11=append(Folder_Path,'Number_DC_RC3.xlsx');
filename12=append(Folder_Path,'Number_DC_RC4.xlsx');
filename13=append(Folder_Path,'Number_DC_RC5.xlsx');
Number_DC_RC2 = xlsread(filename10,1);
Number_DC_RC3 = xlsread(filename11,1);
Number_DC_RC4 = xlsread(filename12,1);
Number_DC_RC5 = xlsread(filename13,1);

%5) Collapse data
Collapse = xlsread(filename5,1);
n_collapse=0;                               % number of realizations that lead to collapse
for i=1:n_real 
    if Collapse(1,i)==1
        n_collapse = n_collapse+1;
    end
end

%6) Demolition due to excess Resdiual Drift data
Residual_Drift = xlsread(filename4,1);
n_RDdemolition=0;                             % number of realizations that lead to demolition due to residual drift
for i=1:n_real 
    if Residual_Drift(1,i)==1 && Collapse(1,i) ~= 1
        n_RDdemolition = n_RDdemolition+1;
    end
end

n_total_reconstruction = n_collapse+n_RDdemolition; % total number of realizations that lead to collapse or demolition
n_repairable = n_real - n_total_reconstruction;   % number of realizations that do not lead to collapse or demolition

%7) Total Economic loss (in $) for each realization
Total_Economic_Loss = xlsread(filename6,1);

% Calculate the normalized economic loss by dividing with total building replacement cost
Economic_Loss_Ratio = Total_Economic_Loss./Replacement_Cost;

% Building repairs can be financed by private loans, public loans or earthquake insurance
% Create a array of elements equal to the number of realizations to
% represent the financing decisions. The decision is represented by
% numerical values
% Value of : 1 represents financing by public loans  
%            2 represents financing by insurance 
%            3 represents private loans
Insurance = 0.16;             % Input in terms of probability of households/buildings that have EQ insurance in the city 

PublicLoan = 0.12;            % Input in terms of probability of households/buildings that belong to 'low-income' category  

Financing_decision = zeros(1,n_real);
% Sample randum numbers for each realization. The building is financed by
% insurance if the number is between 0 and P(Insurance), by public loans if
% the number is between P(Insurance) and the sum of P(Insurance) and
% P(PublicLoan) and by private loans otherwise
for i=1:n_real
    j = rand(1,1);
    if j <= Insurance
        Financing_decision(1,i) = 2;
    elseif j > Insurance && j <= PublicLoan + Insurance
        Financing_decision(1,i) = 1;
    elseif j > PublicLoan + Insurance
        Financing_decision(1,i) = 3;
    end
end

% Create Repair Time Matrix given Repair Class
n_PG = size(Repair_Time_All,1);
Repair_Times_RC2=zeros(n_PG,n_real);
Repair_Times_RC3=zeros(n_PG,n_real);
Repair_Times_RC4=zeros(n_PG,n_real);

% Include Repair Time for each PG in each floor only if the RC exceeds values that hinder functional recovery.
%   RC == 2: Functional Recovery
for i=1:n_PG
    for j=1:n_real 
    if Repair_Class_All(i,j) >= 2  
    Repair_Times_RC2(i,j)= Repair_Time_All(i,j);
    else
    Repair_Times_RC2(i,j) = 0;
    end
    end
end
xlswrite(append(Folder_Path,'Repair_Times_RC2.xlsx'),Repair_Times_RC2)

% Include Repair Time for each PG in each floor only if the RC exceeds values that hinder re-occupancy.
%   RC == 3: Re-occupancy
for i=1:n_PG
    for j=1:n_real 
    if Repair_Class_All(i,j) >= 3 
    Repair_Times_RC3(i,j)= Repair_Time_All(i,j);
    else
    Repair_Times_RC3(i,j) = 0;
    end
    end
end

% Include Repair Time for each PG in each floor only if the RC exceeds values that hinder shelter-in-place recovery state
%   RC == 4: Temporary habitability standards 
for i=1:n_PG
    for j=1:n_real 
    if Repair_Class_All(i,j) >= 4  
    Repair_Times_RC4(i,j)= Repair_Time_All(i,j);
    else
    Repair_Times_RC4(i,j) = 0;
    end
    end
end

% Calculate Repair Time per Repair Sequence (RS)            % The excel files generated will be used by Repair_Sequencing.m script
RS_Repair_Times_RC2=zeros(n_Floors*7,n_real);
RS_Repair_Times_RC3=zeros(n_Floors*7,n_real);
RS_Repair_Times_RC4=zeros(n_Floors*7,n_real);

% RC2
for i=1:n_real
    for j=1:n_Floors
        RS_1=0;
        RS_2=0;
        RS_3=0;
        RS_4=0;
        RS_5=0;
        RS_6=0;
        RS_7=0;
        for k=1:n_PG
            if RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==1
                     RS_1=RS_1+Repair_Times_RC2(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==2
                     RS_2=RS_2+Repair_Times_RC2(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==3
                     RS_3=RS_3+Repair_Times_RC2(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==4
                     RS_4=RS_4+Repair_Times_RC2(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==5
                     RS_5=RS_5+Repair_Times_RC2(k,i);                     
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==6
                     RS_6=RS_6+Repair_Times_RC2(k,i);                            
             elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==7
                     RS_7=RS_7+Repair_Times_RC2(k,i);        
            end     
        end                                  
            p=(j-1);     
            RS_Repair_Times_RC2(p*7+1,i)=RS_1;
            RS_Repair_Times_RC2(p*7+2,i)=RS_2;       
            RS_Repair_Times_RC2(p*7+3,i)=RS_3;                
            RS_Repair_Times_RC2(p*7+4,i)=RS_4;     
            RS_Repair_Times_RC2(p*7+5,i)=RS_5;         
            RS_Repair_Times_RC2(p*7+6,i)=RS_6;         
            RS_Repair_Times_RC2(p*7+7,i)=RS_7;     
    end 
end 

% Modification for elevator repair time 
% Elevators are modelled  only on the first floor of the structure and hence the repair time is
% concentrated only on the first floor. Since elevator rail and door 
% repairs will be carried out over all floors of the building, we
% equally divide the elevator repair time over all floors of the building. 

for i=1:n_real  
    RT_Elevator = RS_Repair_Times_RC2(6,i);         %The elevator rail repair is part of the 6th repair sequence
    RT_Elevator_per_floor = RT_Elevator/n_Floors;
    
    for j=1:n_Floors
        k=j-1;
        RS_Repair_Times_RC2(7*k+6,i)=RT_Elevator_per_floor ;
    end
end
xlswrite(append(Folder_Path,'RS_Repair_Times_RC2.xlsx'),RS_Repair_Times_RC2)

% RC3
for i=1:n_real
    for j=1:n_Floors
        RS_1=0;
        RS_2=0;
        RS_3=0;
        RS_4=0;
        RS_5=0;
        RS_6=0;
        RS_7=0;
        for k=1:n_PG
            if RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==1
                     RS_1=RS_1+Repair_Times_RC3(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==2
                     RS_2=RS_2+Repair_Times_RC3(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==3
                     RS_3=RS_3+Repair_Times_RC3(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==4
                     RS_4=RS_4+Repair_Times_RC3(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==5
                     RS_5=RS_5+Repair_Times_RC3(k,i);                     
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==6
                     RS_6=RS_6+Repair_Times_RC3(k,i);                            
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==7
                     RS_7=RS_7+Repair_Times_RC3(k,i);        
            end        
        end                                  
            p=(j-1);     
            RS_Repair_Times_RC3(p*7+1,i)=RS_1;
            RS_Repair_Times_RC3(p*7+2,i)=RS_2;       
            RS_Repair_Times_RC3(p*7+3,i)=RS_3;                
            RS_Repair_Times_RC3(p*7+4,i)=RS_4;     
            RS_Repair_Times_RC3(p*7+5,i)=RS_5;         
            RS_Repair_Times_RC3(p*7+6,i)=RS_6;         
            RS_Repair_Times_RC3(p*7+7,i)=RS_7;     
    end 
end 
xlswrite(append(Folder_Path,'RS_Repair_Times_RC3.xlsx'),RS_Repair_Times_RC3)

% RC4
for i=1:n_real
    for j=1:n_Floors
        RS_1=0;
        RS_2=0;
        RS_3=0;
        RS_4=0;
        RS_5=0;
        RS_6=0;
        RS_7=0;
        for k=1:n_PG
            if RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==1
                     RS_1=RS_1+Repair_Times_RC4(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==2
                     RS_2=RS_2+Repair_Times_RC4(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==3
                     RS_3=RS_3+Repair_Times_RC4(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==4
                     RS_4=RS_4+Repair_Times_RC4(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==5
                     RS_5=RS_5+Repair_Times_RC4(k,i);                     
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==6
                     RS_6=RS_6+Repair_Times_RC4(k,i);                            
             elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==7
                     RS_7=RS_7+Repair_Times_RC4(k,i);        
            end     
        end                                  
            p=(j-1);     
            RS_Repair_Times_RC4(p*7+1,i)=RS_1;
            RS_Repair_Times_RC4(p*7+2,i)=RS_2;       
            RS_Repair_Times_RC4(p*7+3,i)=RS_3;                
            RS_Repair_Times_RC4(p*7+4,i)=RS_4;     
            RS_Repair_Times_RC4(p*7+5,i)=RS_5;         
            RS_Repair_Times_RC4(p*7+6,i)=RS_6;         
            RS_Repair_Times_RC4(p*7+7,i)=RS_7;     
    end 
end 
xlswrite(append(Folder_Path,'RS_Repair_Times_RC4.xlsx'),RS_Repair_Times_RC4)

% Calculate the number of damaged components trigerring RC2 per floor per repair sequence
RS_Number_DC_RC2 = zeros(n_Floors*7,n_real);
for i=1:n_real
    for j=1:n_Floors
        RS_1=0;
        RS_2=0;
        RS_3=0;
        RS_4=0;
        RS_5=0;
        RS_6=0;
        RS_7=0;
        for k=1:n_PG
            if RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==1
                     RS_1=RS_1+Number_DC_RC2(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==2
                     RS_2=RS_2+Number_DC_RC2(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==3
                     RS_3=RS_3+Number_DC_RC2(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==4
                     RS_4=RS_4+Number_DC_RC2(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==5
                     RS_5=RS_5+Number_DC_RC2(k,i);                     
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==6
                     RS_6=RS_6+Number_DC_RC2(k,i);                            
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==7
                     RS_7=RS_7+Number_DC_RC2(k,i);        
            end
        end                                  
            p=(j-1);     
            RS_Number_DC_RC2(p*7+1,i)=RS_1;
            RS_Number_DC_RC2(p*7+2,i)=RS_2;       
            RS_Number_DC_RC2(p*7+3,i)=RS_3;                
            RS_Number_DC_RC2(p*7+4,i)=RS_4;     
            RS_Number_DC_RC2(p*7+5,i)=RS_5;         
            RS_Number_DC_RC2(p*7+6,i)=RS_6;         
            RS_Number_DC_RC2(p*7+7,i)=RS_7;     
    end 
end 
xlswrite(append(Folder_Path,'RS_Number_DC_RC2.xlsx'),RS_Number_DC_RC2)

% Calculate the number of damaged components trigerring RC3 per floor per repair sequence
RS_Number_DC_RC3 = zeros(n_Floors*7,n_real);
for i=1:n_real
    for j=1:n_Floors
        RS_1=0;
        RS_2=0;
        RS_3=0;
        RS_4=0;
        RS_5=0;
        RS_6=0;
        RS_7=0;
        for k=1:n_PG
            if RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==1
                     RS_1=RS_1+Number_DC_RC3(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==2
                     RS_2=RS_2+Number_DC_RC3(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==3
                     RS_3=RS_3+Number_DC_RC3(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==4
                     RS_4=RS_4+Number_DC_RC3(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==5
                     RS_5=RS_5+Number_DC_RC3(k,i);                     
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==6
                     RS_6=RS_6+Number_DC_RC3(k,i);                            
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==7
                     RS_7=RS_7+Number_DC_RC3(k,i);        
            end
        end                                  
            p=(j-1);     
            RS_Number_DC_RC3(p*7+1,i)=RS_1;
            RS_Number_DC_RC3(p*7+2,i)=RS_2;       
            RS_Number_DC_RC3(p*7+3,i)=RS_3;                
            RS_Number_DC_RC3(p*7+4,i)=RS_4;     
            RS_Number_DC_RC3(p*7+5,i)=RS_5;         
            RS_Number_DC_RC3(p*7+6,i)=RS_6;         
            RS_Number_DC_RC3(p*7+7,i)=RS_7;     
    end 
end 
xlswrite(append(Folder_Path,'RS_Number_DC_RC3.xlsx'),RS_Number_DC_RC3)

% Calculate the number of damaged components trigerring RC4 per floor per repair sequence
RS_Number_DC_RC4 = zeros(n_Floors*7,n_real);
for i=1:n_real
    for j=1:n_Floors
        RS_1=0;
        RS_2=0;
        RS_3=0;
        RS_4=0;
        RS_5=0;
        RS_6=0;
        RS_7=0;
        for k=1:n_PG
            if RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==1
                     RS_1=RS_1+Number_DC_RC4(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==2
                     RS_2=RS_2+Number_DC_RC4(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==3
                     RS_3=RS_3+Number_DC_RC4(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==4
                     RS_4=RS_4+Number_DC_RC4(k,i);
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==5
                     RS_5=RS_5+Number_DC_RC4(k,i);                     
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==6
                     RS_6=RS_6+Number_DC_RC4(k,i);                            
            elseif RS_Story_Identifier(k,2) ==j && RS_Story_Identifier(k,1) ==7
                     RS_7=RS_7+Number_DC_RC4(k,i);        
            end
        end                                  
            p=(j-1);     
            RS_Number_DC_RC4(p*7+1,i)=RS_1;
            RS_Number_DC_RC4(p*7+2,i)=RS_2;       
            RS_Number_DC_RC4(p*7+3,i)=RS_3;                
            RS_Number_DC_RC4(p*7+4,i)=RS_4;     
            RS_Number_DC_RC4(p*7+5,i)=RS_5;         
            RS_Number_DC_RC4(p*7+6,i)=RS_6;         
            RS_Number_DC_RC4(p*7+7,i)=RS_7;     
    end 
end 
xlswrite(append(Folder_Path,'RS_Number_DC_RC4.xlsx'),RS_Number_DC_RC4)

% Since the damaged components in RC5 will only belong to structural repair
% sequence, we do not do this procedure for RC5



%% Calculating Impeding Factors 
% The abbreviation 'IF' is used for Impeding Factor. 
% Rather than simply using median estimates, we sample all impeding factors for every realization 
% Impeding factors to contractor mobilization, engineering and permitting 
% are weighted as a function of the repair class observed in each storey,

% The Impeding factor for contractor moblization depends on the repair
% sequence. Hence we need to know the maximum repair class triggered in each floor for each repair sequence

% Calculate the max repair class in each floor for each repair sequence
RC_Max_RS1=zeros(n_Floors,n_real);
RC_Max_RS2=zeros(n_Floors,n_real);
RC_Max_RS3=zeros(n_Floors,n_real);
RC_Max_RS4=zeros(n_Floors,n_real);
RC_Max_RS5=zeros(n_Floors,n_real);
RC_Max_RS6=zeros(n_Floors,n_real);
RC_Max_RS7=zeros(n_Floors,n_real);

for i=1:n_real
    for j=1:n_Floors
    RC_RS1=0;RC_RS2=0;RC_RS3=0;RC_RS4=0;RC_RS5=0;RC_RS6=0;RC_RS7=0; p=1;
        for k=1:n_PG
            if RS_Story_Identifier(k,1) ==1 && RS_Story_Identifier(k,2) ==j
                RC_RS1(p) = Repair_Class_All(k,i);
                p=p+1;
                RC_Max_RS1(j,i) = max(RC_RS1);
            elseif RS_Story_Identifier(k,1) ==2 && RS_Story_Identifier(k,2) ==j
                RC_RS2(p) = Repair_Class_All(k,i);
                p=p+1;
                RC_Max_RS2(j,i) = max(RC_RS2);
            elseif RS_Story_Identifier(k,1) ==3 && RS_Story_Identifier(k,2) ==j
                RC_RS3(p) = Repair_Class_All(k,i);
                p=p+1;
                RC_Max_RS3(j,i) = max(RC_RS3);
            elseif RS_Story_Identifier(k,1) ==4 && RS_Story_Identifier(k,2) ==j
                RC_RS4(p) = Repair_Class_All(k,i);
                p=p+1;
                RC_Max_RS4(j,i) = max(RC_RS4);
            elseif RS_Story_Identifier(k,1) ==5 && RS_Story_Identifier(k,2) ==j
                RC_RS5(p) = Repair_Class_All(k,i);
                p=p+1;
                RC_Max_RS5(j,i) = max(RC_RS5);
            elseif RS_Story_Identifier(k,1) ==6 && RS_Story_Identifier(k,2) ==j
                RC_RS6(p) = Repair_Class_All(k,i);
                p=p+1;
                RC_Max_RS6(j,i) = max(RC_RS6);
            elseif RS_Story_Identifier(k,1) ==7 && RS_Story_Identifier(k,2) ==j
                RC_RS7(p) = Repair_Class_All(k,i);
                p=p+1;
                RC_Max_RS7(j,i) = max(RC_RS7);
            end
        end
    end
end   

for i=1:n_real
    RC_Max_RS1_0(1,i)=nnz(RC_Max_RS1(:,i)==0);
    RC_Max_RS1_1(1,i)=nnz(RC_Max_RS1(:,i)==1);
    RC_Max_RS1_2(1,i)=nnz(RC_Max_RS1(:,i)==2);    
    RC_Max_RS1_3(1,i)=nnz(RC_Max_RS1(:,i)==3);
    RC_Max_RS1_4(1,i)=nnz(RC_Max_RS1(:,i)>=4);
end 

for i=1:n_real
    RC_Max_RS2_0(1,i)=nnz(RC_Max_RS2(:,i)==0);
    RC_Max_RS2_1(1,i)=nnz(RC_Max_RS2(:,i)==1);
    RC_Max_RS2_2(1,i)=nnz(RC_Max_RS2(:,i)==2);    
    RC_Max_RS2_3(1,i)=nnz(RC_Max_RS2(:,i)==3);
    RC_Max_RS2_4(1,i)=nnz(RC_Max_RS2(:,i)>=4); 
end  

for i=1:n_real
    RC_Max_RS3_0(1,i)=nnz(RC_Max_RS3(:,i)==0);
    RC_Max_RS3_1(1,i)=nnz(RC_Max_RS3(:,i)==1);
    RC_Max_RS3_2(1,i)=nnz(RC_Max_RS3(:,i)==2);    
    RC_Max_RS3_3(1,i)=nnz(RC_Max_RS3(:,i)==3);   
    RC_Max_RS3_4(1,i)=nnz(RC_Max_RS3(:,i)>=4);  
end 

for i=1:n_real
    RC_Max_RS4_0(1,i)=nnz(RC_Max_RS4(:,i)==0);
    RC_Max_RS4_1(1,i)=nnz(RC_Max_RS4(:,i)==1);
    RC_Max_RS4_2(1,i)=nnz(RC_Max_RS4(:,i)==2);    
    RC_Max_RS4_3(1,i)=nnz(RC_Max_RS4(:,i)>=3);     
end 

for i=1:n_real
    RC_Max_RS5_0(1,i)=nnz(RC_Max_RS5(:,i)==0);
    RC_Max_RS5_1(1,i)=nnz(RC_Max_RS5(:,i)==1);
    RC_Max_RS5_2(1,i)=nnz(RC_Max_RS5(:,i)==2);    
    RC_Max_RS5_3(1,i)=nnz(RC_Max_RS5(:,i)>=3);     
end 

for i=1:n_real
    RC_Max_RS6_0(1,i)=nnz(RC_Max_RS6(:,i)==0);
    RC_Max_RS6_1(1,i)=nnz(RC_Max_RS6(:,i)==1);
    RC_Max_RS6_2(1,i)=nnz(RC_Max_RS6(:,i)==2);    
    RC_Max_RS6_3(1,i)=nnz(RC_Max_RS6(:,i)>=3);     
end 

for i=1:n_real
    RC_Max_RS7_0(1,i)=nnz(RC_Max_RS7(:,i)==0);
    RC_Max_RS7_1(1,i)=nnz(RC_Max_RS7(:,i)==1);
    RC_Max_RS7_2(1,i)=nnz(RC_Max_RS7(:,i)==2);    
    RC_Max_RS7_3(1,i)=nnz(RC_Max_RS7(:,i)==3);
    RC_Max_RS7_4(1,i)=nnz(RC_Max_RS7(:,i)>=4); 
end 

RC_Max_All = zeros(1,n_real);
for i=1:n_real
    RC_Max_All(1,i) = max(Repair_Class_All(:,i));
end

% Calculate the number of damaged curtain wall units with glass falling out i.e. RC=3 (used to calculate the building stabilization time)
n_RS3_RC3 = zeros(1,n_real);
for i=1:n_real
    k=0;
    for j=1:n_PG
        if RS_Story_Identifier(j,1) == 3
        k = k + Number_DC_RC3(j,i);
        end
    end
    n_RS3_RC3(1,i) = k; 
end 

IF=zeros(5,n_real);      % One row each for: Inspection, Financing, Engineering, Permitting and Building Stabilization / Demolition           
Utility_DT = zeros(3,n_real);
IF_CM = zeros(7,n_real); % The impeding factor for Contractor Mobilization (CM) depends on the repair sequence

for i=1:n_real
    
    RC_RS1 = max(RC_Max_RS1(:,i)); 
    RC_RS2 = max(RC_Max_RS2(:,i));
    RC_RS3 = max(RC_Max_RS3(:,i));
    RC_RS4 = max(RC_Max_RS4(:,i));
    RC_RS5 = max(RC_Max_RS5(:,i));
    RC_RS6 = max(RC_Max_RS6(:,i));
    RC_RS7 = max(RC_Max_RS7(:,i));
    CM_RC_Max_RS1_1 = RC_Max_RS1_1(1,i)/n_Floors;
    CM_RC_Max_RS1_2 = RC_Max_RS1_2(1,i)/n_Floors;
    CM_RC_Max_RS1_3 = RC_Max_RS1_3(1,i)/n_Floors; 
    CM_RC_Max_RS1_4 = RC_Max_RS1_4(1,i)/n_Floors; 
    
    CM_RC_Max_RS2_1 = RC_Max_RS2_1(1,i)/n_Floors;
    CM_RC_Max_RS2_2 = RC_Max_RS2_2(1,i)/n_Floors;
    CM_RC_Max_RS2_3 = RC_Max_RS2_3(1,i)/n_Floors; 
    CM_RC_Max_RS2_4 = RC_Max_RS2_4(1,i)/n_Floors;
 
    CM_RC_Max_RS3_1 = RC_Max_RS3_1(1,i)/n_Floors;
    CM_RC_Max_RS3_2 = RC_Max_RS3_2(1,i)/n_Floors; 
    CM_RC_Max_RS3_3 = RC_Max_RS3_3(1,i)/n_Floors; 
    CM_RC_Max_RS3_4 = RC_Max_RS3_4(1,i)/n_Floors; 
    
    CM_RC_Max_RS4_1 = RC_Max_RS4_1(1,i)/n_Floors;
    CM_RC_Max_RS4_2 = RC_Max_RS4_2(1,i)/n_Floors; 
    CM_RC_Max_RS4_3 = RC_Max_RS4_3(1,i)/n_Floors;    
    
    CM_RC_Max_RS5_1 = RC_Max_RS5_1(1,i)/n_Floors;
    CM_RC_Max_RS5_2 = RC_Max_RS5_2(1,i)/n_Floors; 
    CM_RC_Max_RS5_3 = RC_Max_RS5_3(1,i)/n_Floors; 
   
    CM_RC_Max_RS6_1 = RC_Max_RS6_1(1,i)/n_Floors;
    CM_RC_Max_RS6_2 = RC_Max_RS6_2(1,i)/n_Floors; 
    CM_RC_Max_RS6_3 = RC_Max_RS6_3(1,i)/n_Floors;  

    CM_RC_Max_RS7_1 = RC_Max_RS7_1(1,i)/n_Floors;
    CM_RC_Max_RS7_2 = RC_Max_RS7_2(1,i)/n_Floors; 
    CM_RC_Max_RS7_3 = RC_Max_RS7_3(1,i)/n_Floors;
    CM_RC_Max_RS7_4 = RC_Max_RS7_4(1,i)/n_Floors;
     
    % Inspection
            if RC_Max_All(1,i) == 0
            IF(1,i)=0; 
            elseif RC_Max_All(1,i) >= 1
            IF(1,i)= logninv(rand(1,1),log(5),0.54);  
            end 
            
    % Engineering  & Permitting
            if RC_RS1 ==0
            IF(3,i)=0;   
            IF(4,i)=0;
            else 
            IF(3,i)=CM_RC_Max_RS1_1*logninv(rand(1,1),log(42),0.4) + (CM_RC_Max_RS1_2+CM_RC_Max_RS1_3+CM_RC_Max_RS1_4)*logninv(rand(1,1),log(84),0.4); 
            IF(4,i)=CM_RC_Max_RS1_1*logninv(rand(1,1),log(7),0.86) + (CM_RC_Max_RS1_2+CM_RC_Max_RS1_3+CM_RC_Max_RS1_4)*logninv(rand(1,1),log(56),0.32);
            end
            
     % Financing 
            % No delay if there is no damage 
            if RC_Max_All(1,i) == 0 
                Financing_decision(1,i) = 0;
            end
            
            if Financing_decision(1,i) == 2                                 % Financing by insurance
                if Economic_Loss_Ratio(1,i) <= 0.05                        % ELR less than deductible and 5% --> deductible is self-financed
                    IF(2,i)= 0;
                elseif Economic_Loss_Ratio(1,i) > 0.05 && Economic_Loss_Ratio(1,i) <= 0.1      % ELR less than deductible but greater than 2.5% --> deductible financed by private loans
                    IF(2,i)= 0.5 * logninv(rand(1,1),log(105),0.68);   
                elseif Economic_Loss_Ratio(1,i) > 0.1                       % ELR > deductible --> financing delay max of insurance delay & delay due to deductible financing by private loans
                    IF(2,i)= max (0.5 * logninv(rand(1,1),log(105),0.68),...
                        1.00 * logninv(rand(1,1),log(42),1.11));
                end
            elseif Financing_decision(1,i) == 1                             % Financing by public loans. For low income occupants, self funding of repairs is not considered an option
                if Economic_Loss_Ratio(1,i) <= 0.1
                    IF(2,i)= 0.5 * logninv(rand(1,1),log(336),0.57);
                elseif Economic_Loss_Ratio(1,i) > 0.1
                    IF(2,i)= 1.0 * logninv(rand(1,1),log(336),0.57);
                end
            elseif Financing_decision(1,i) == 3                             % Financing by private loans
                if Economic_Loss_Ratio(1,i) <= 0.05                        % ELR <= 5% --> repairs are self funded
                    IF(2,i)= 0;
                elseif Economic_Loss_Ratio(1,i) > 0.05 && Economic_Loss_Ratio(1,i) <= 0.1      % ELR >5% but less than 10% --> short delay for private loans
                    IF(2,i)= 0.5 * logninv(rand(1,1),log(105),0.68);
                elseif Economic_Loss_Ratio(1,i) > 0.1
                    IF(2,i)= 1.0 * logninv(rand(1,1),log(105),0.68);
                end
            elseif Financing_decision(1,i) == 0
                IF(2,i) = 0;
            end 
            
     % Contractor mobilization      
            if RC_RS1 ==0
            IF_CM(1,i)=0;   
            else
            IF_CM(1,i)=CM_RC_Max_RS1_1*logninv(rand(1,1),log(14.4*7),0.392) + (CM_RC_Max_RS1_2+CM_RC_Max_RS1_3+CM_RC_Max_RS1_4)*logninv(rand(1,1),log(21.5*7),0.334);
            end

            if RC_RS2 ==0
            IF_CM(2,i)=0;   
            else
            IF_CM(2,i)=CM_RC_Max_RS2_1*logninv(rand(1,1),log(7.2*7),0.347) + (CM_RC_Max_RS2_2+CM_RC_Max_RS2_3+CM_RC_Max_RS2_4)*logninv(rand(1,1),log(17.5*7),0.429);
            end
            
            if RC_RS3 ==0
            IF_CM(3,i)=0;   
            else
            IF_CM(3,i)=CM_RC_Max_RS3_1*logninv(rand(1,1),log(13.1*7),0.527) + (CM_RC_Max_RS3_2+CM_RC_Max_RS3_3+CM_RC_Max_RS3_4)*logninv(rand(1,1),log(20.8*7),0.8125);
            end
            
            if RC_RS4 ==0
            IF_CM(4,i)=0;   
            else
            IF_CM(4,i)=CM_RC_Max_RS4_1*logninv(rand(1,1),log(12.2*7),0.467) + (CM_RC_Max_RS4_2+CM_RC_Max_RS4_3)*logninv(rand(1,1),log(19.3*7),0.855);
            end
            
            if RC_RS5 ==0
            IF_CM(5,i)=0;   
            else
            IF_CM(5,i)= CM_RC_Max_RS5_1*logninv(rand(1,1),log(8.6*7),0.43) + (CM_RC_Max_RS5_2+CM_RC_Max_RS5_3)*logninv(rand(1,1),log(11.3*7),0.3);
            end
            
            if RC_RS6 ==0
            IF_CM(6,i)=0;   
            else          
            IF_CM(6,i)= logninv(rand(1,1),log(27.5*7),0.485) ;         % Since PG for elevators are located only on the 1st floor, the weighted average method cannot be used  
            end
            
            if RC_RS7 ==0
            IF_CM(7,i)=0;   
            else
            IF_CM(7,i)= CM_RC_Max_RS7_1*logninv(rand(1,1),log(8.2*7),0.39) + (CM_RC_Max_RS7_2+CM_RC_Max_RS7_3+CM_RC_Max_RS7_4)*logninv(rand(1,1),log(17.3*7),0.936);
            end
            
    % Time to ensure building stability
            
            % Time for emergency structural repairs
            n_RC5 = sum(Number_DC_RC5(:,i));            % Number of structural components in RC5
            % The repair time is considered as 6 days for up to 3 damaged components,
            % 4 days for more than 7 damaged components, and calculated by the
            % following equation for qty between 3 and 7: -0.5*n_RC5 + 7.5   
            %One crews is considered to be working simultanesouly for less
            %than 7 damaged components and two crews for 7 or more damaged components 
            if n_RC5 == 0
            Stabilization_RS1 = 0;
            elseif n_RC5 <= 3
                TempRT = logninv(rand(1,1),log(6),0.4); 
                Stabilization_RS1 = (n_RC5 * TempRT ) / 1 ;           % Considering 1 crews working simulataneously for n_RC5 <7
            
            elseif n_RC5 > 3 && n_RC5 < 7
                TempRT_WODisp =  -0.5*n_RC5 + 7.5 ;
                TempRT = logninv(rand(1,1),log(TempRT_WODisp),0.4); 
                Stabilization_RS1 = (n_RC5 * TempRT ) / 1 ;   % Considering 1 crews working simulataneously for n_RC5 <7
            
            elseif n_RC5 >= 7
                TempRT = logninv(rand(1,1),log(4),0.4); 
                Stabilization_RS1 = (n_RC5 * TempRT ) / 2 ;           % Considering 2 crews working simulataneously for n_RC5 >=7
            end
            
            % Calculate time to remove damaged curtain walls in RC3
            % The time to remove one panel is considered as 0.143 days for up to 20 damaged components,
            % 0.076 days for more than 100 damaged components, and calculated by the
            % following equation for qty between 20 and 100: -0.0008375*n_RS3_RC3 + 0.15975   
            %One crews is considered to be working simultanesouly for less
            %than 7 damaged components and two crews for 7 or more damaged components 
            % Mobilization time of 5 days & time to remove each damaged panel is considered as 0.1429 days
            % i.e. 1.4hours per panel considering 10 hour workdays
            if n_RS3_RC3(1,i) == 0
                Stabilization_RS3 =0;
            elseif n_RS3_RC3(1,i) <= 20
                TempRT = logninv(rand(1,1),log(0.1429),0.4);
                Stabilization_RS3 = n_RS3_RC3(1,i) * TempRT;
                
            elseif n_RS3_RC3(1,i) > 20 && n_RS3_RC3(1,i) <100
                TempRT_WODisp =  -0.0008375*n_RS3_RC3(1,i) + 0.15975 ;
                TempRT = logninv(rand(1,1),log(TempRT_WODisp),0.4);
                Stabilization_RS3 = n_RS3_RC3(1,i) * TempRT;   
                
            elseif n_RS3_RC3(1,i) >= 20
                TempRT = logninv(rand(1,1),log(0.076),0.4) / 2;
                Stabilization_RS3 = n_RS3_RC3(1,i) * TempRT;    
            end
            IF(5,i) = max(Stabilization_RS1 , Stabilization_RS3);
            
    % Utility downtime
            % Electrical systems
            Utility_DT(1,i) = logninv(rand(1,1),log(3),1.2);  
            % Natural gas systems
            Utility_DT(2,i) = logninv(rand(1,1),log(10),1); 
            % Water systems
            Utility_DT(3,i) = logninv(rand(1,1),log(4),0.55); 
end

% Accounting for collapse and residual drift 
for i=1:n_real
    if Residual_Drift(1,i) == 1 || Collapse(1,i) == 1
        IF(:,i) = 0;            % Make impeding factors = 0 and only consider demolition 
        IF_CM(:,i)=0;
        Utility_DT(:,i)=0;
        
        IF(5,i)=logninv(rand(1,1),log(63.6*7),0.67);          % Governing delay considering demolition and financing as per Marquis(2015), and engineering as per REDi (2013)
    end
end

xlswrite(append(Folder_Path,'AllReal_ImpedingFactors.xlsx'),IF);
xlswrite(append(Folder_Path,'IF_CM.xlsx'),IF_CM);
xlswrite(append(Folder_Path,'Utility_Downtime.xlsx'),Utility_DT);
 
% Impeding factor summary based on repair sequence
IF_Summary_RS = zeros(7,n_real);
for i=1:n_real
    for k=1:7
           Path_1=IF(1,i)+IF(2,i);              % Financing
           Path_2=IF(1,i)+IF(3,i)+IF(4,i);      % Engineering + Permitting
           Path_3=IF(1,i)+IF_CM(k,i);           % Contractor Mobilization
           Path_4=IF(1,i)+IF(5,i);              % Building Stabilization Time or Demolition time
           
        IF_Summary_RS(k,i)=max([Path_1 Path_2 Path_3 Path_4]);
    end
end
xlswrite(append(Folder_Path,'IF_Summary_RS.xlsx'),IF_Summary_RS)

%% Plotting the summary of impeding factors

% Separate the Imeding factors for collapse and demolition
IF_nocollapse = IF;
IF_reconstruction = zeros(1,n_real);
for i=1:n_real
    if Residual_Drift(1,i) == 1 || Collapse(1,i) == 1
        IF_reconstruction(1,i) = IF(5,i);
        IF_nocollapse(:,i) = 0;
    end
end

%% Peform repair sequencing operation by calling the repair sequencing function for each repair class

% Determine the phases
% Col1: first floor of each phase
% Col2: last floor of each phase
% Number of rows = Number of phases
Phases = zeros(ceil(n_Floors_AG/3)+1,2);        % 1 phase for all floors BG and then floors abvoe grade in groups of three floors
Phases(1,1) = 1;
Phases(1,2) = n_Floors-n_Floors_AG;
for i=2:size(Phases,1)
    Phases(i,1)=Phases(i-1,2)+1;
    Phases(i,2) = min(Phases(i,1)+2,n_Floors);
end

[Downtime_Median_RC2 , DT_Range_RC2] = Repair_Sequencing(Folder_Path,'RC2',n_real,n_Floors,n_Floors_AG,Phases);
%[Downtime_Median_RC3 , DT_Range_RC3] = Repair_Sequencing(Folder_Path,'RC3',n_real,n_Floors,n_Floors_AG,Phases);
%[Downtime_Median_RC4 , DT_Range_RC4] = Repair_Sequencing(Folder_Path,'RC4',n_real,n_Floors,n_Floors_AG,Phases);

% Write the median stepping curve in an excel file
% Later, we plot the median stepping curves for all intensities, hence we
% designate an excel range to each intensity
% Col A1 to E1: 2014 NSHM (100 yr to 4975 yr)
% Col F1 to J1: 2018 NSHM (100 yr to 4975 yr)
% Col K1: M9 scenario


% Write the range in variation of downtime in an excel file for each
% intensity (Used to plot robustness performance measure)
rowNames = {'Less than 4 months','Between 4 and 12 months','Between 12 and 24 months','Between 24 and 36 months','Greater than 36 months'};
Downtime_ConfidenceLevels_RC2 = array2table(DT_Range_RC2,'RowNames',rowNames);
% Downtime_ConfidenceLevels_RC3 = array2table(DT_Range_RC3,'RowNames',rowNames);
%writetable(Downtime_ConfidenceLevels_RC3,append(Folder_Path,'Downtime_ConfidenceLevels.xlsx'),'Sheet',2,'WriteRowNames',true)
% Downtime_ConfidenceLevels_RC4 = array2table(DT_Range_RC4,'RowNames',rowNames);
%writetable(Downtime_ConfidenceLevels_RC4,append(Folder_Path,'Downtime_ConfidenceLevels.xlsx'),'Sheet',3,'WriteRowNames',true)

%% Plotting the probablity of observing the recovery-based limit states as defined in Burton et al (2015)
% Only for For M9 scenario (since collapse is not included in SP3 analysis for M9)
% if int == 11
%     n_collapse = 200;
%     n_RDcollapse = n_RDcollapse - n_collapse;
% end
% 

% Currently exterior facade components that hinder stability are not tagged
% with a repair class of 5 because these do not need to "repaired" for
% stability or shelter-in-place, but if damaged panels exceeds 50%, then a
% cordon may be triggered around the building  
RS3_Stability = xlsread(append(Folder_Path,'RS3_Stability.xlsx'),1);

n_RC1=0;    % Number of realizations where inspection is triggerred
n_RC2=0;    % Number of realizations where functinal recovery is triggerred
n_RC3=0;    % Number of realizations where reoccupancy is triggerred
n_RC4=0;    % Number of realizations where RC4 triggerred i.e. shelter-in-place will be not be possible
n_RC5=0;    % Number of realizations where Building cordon is triggerred
% The
for i=1:n_real
    if RC_Max_All(1,i) == 5 || RS3_Stability(1,i) == 5
        n_RC5 = n_RC5+1;
    elseif RC_Max_All(1,i) == 4
        n_RC4 = n_RC4+1;
    elseif RC_Max_All(1,i) == 3
        n_RC3 = n_RC3+1;
    elseif RC_Max_All(1,i) == 2
        n_RC2 = n_RC2+1;
    elseif RC_Max_All(1,i) == 1
        n_RC1 = n_RC1+1;
    end
end      

n_RC0 = n_real - (n_collapse + n_RDdemolition + n_RC5 + n_RC4 + n_RC3 + n_RC2 + n_RC1);
Probabilities = [n_RC0+n_RC1;n_RC2; n_RC3; n_RC4; n_RC5+ n_RDdemolition+ n_collapse] ./n_real*100;
rowNames1 = {'Full recovery not acheieved','Functional recovery not acheieved','Reoccupancy not acheieved','Shelter-in-place not acheieved','Collapse or Demolition or Stability hindered'};
Probabilities_LimitStates = array2table(Probabilities,'RowNames',rowNames1);


%% PLOTTING DISAGGREGATION OF DATA
% Plotting the percent of realizations where a repair class is triggerred

% All components are divided in two twelve groups to understand the
% criticality of each component to the total downtime
Groupnames = categorical({'Shear wall','Slab-column connections','Hot & cold water piping'...
    'Wall partitions & finishing','Suspended ceiling','HVAC','Fire sprinkler system'...
    'Mechanical equipment','Electrical equipment','Curtain walls','Elevator','Staircase'});

% The RCplotting function gives us two outputs:
% The first output term gives us the the percentage of realizations for which damage
% to the component triggers each recovery state for that component group
% The second output term gives us the mean repair time for each component group 
% across all floors and across all realizations

[G1a_RC, G1a_RT] = RCplotting(Folder_Path,n_real,n_Floors,n_PG,1,'1a',Phases);

[G1b_RC, G1b_RT] = RCplotting(Folder_Path,n_real,n_Floors,n_PG,1,'1b',Phases);

[G2a_RC, G2a_RT] = RCplotting(Folder_Path,n_real,n_Floors,n_PG,2,'2a',Phases);

[G2b_RC, G2b_RT] = RCplotting(Folder_Path,n_real,n_Floors,n_PG,2,'2b',Phases);

[G2c_RC, G2c_RT] = RCplotting(Folder_Path,n_real,n_Floors,n_PG,2,'2c',Phases);

[G2d_RC, G2d_RT] = RCplotting(Folder_Path,n_real,n_Floors,n_PG,2,'2d',Phases);

[G2e_RC, G2e_RT] = RCplotting(Folder_Path,n_real,n_Floors,n_PG,2,'2e',Phases);

[G3_RC, G3_RT] = RCplotting(Folder_Path,n_real,n_Floors,n_PG,3,3,Phases);

[G4_RC, G4_RT] = RCplotting(Folder_Path,n_real,n_Floors,n_PG,4,4,Phases);

[G5_RC, G5_RT] = RCplotting(Folder_Path,n_real,n_Floors,n_PG,5,5,Phases);

[G6_RC, G6_RT] = RCplotting(Folder_Path,n_real,n_Floors,n_PG,6,6,Phases);

[G7_RC, G7_RT] = RCplotting(Folder_Path,n_real,n_Floors,n_PG,7,7,Phases);


% Plot the recovery states triggered by component damage
RC_Deaggregation = [G1a_RC;G1b_RC;G2a_RC;G2b_RC;G2c_RC;G2d_RC;G2e_RC;G4_RC;G5_RC;G3_RC;G6_RC;G7_RC];

xlswrite('RC_deaggregation.xlsx',RC_Deaggregation,1)

% To plot the disaggregation of mean downtime 
% 2D plot -  stacks and grouped
Mean_IF = zeros(5,1);
Mean_IF_CM = zeros(7,1);
% Deaggregation data is plotted only for repairable realizatioms
for j=1:5
    Mean_IF(j) = sum(IF_nocollapse(j,:)) / (n_repairable);
end
for j=1:7
    Mean_IF_CM(j) = sum(IF_CM(j,:)) / (n_repairable);
end

Deaggregation_data = zeros(17,5);

Deaggregation_data(:,1) = Mean_IF(1); % Inspection
for j=1:4
    p=j-1;
    Deaggregation_data(4*p+1,2) = Mean_IF(2);       % Financing
    Deaggregation_data(4*p+2,2) = Mean_IF(3);       % Engineering
    Deaggregation_data(4*p+2,3) = Mean_IF(4);       % Permitting
    Deaggregation_data(4*p+3,2) = Mean_IF(5);       % Building Stabilization
end
    
% Contractor moblization impeding factors
Deaggregation_data(4,2) = Mean_IF_CM(1);         % Path A: CM for Path 1
Deaggregation_data(8,2) = Mean_IF_CM(3);              % Path B: CM for RS3
Deaggregation_data(12,2) = Mean_IF_CM(6);             % Path C: CM for RS6
Deaggregation_data(16,2) = Mean_IF_CM(7);              % Path D: CM for RS7


% Repair Time
% Divide repair time by the number of repairable realizations
Deaggregation_data(1:4,4) = (G1a_RT + G1b_RT)  /  n_repairable;                      % Path A: Repair time for RS1
Deaggregation_data(1:4,5) = (  max([G2a_RT+G2b_RT+G2c_RT+G2d_RT+G2e_RT,G4_RT,G5_RT]) )  /  n_repairable;   % Path A: Repair time for RS2, RS4 and RS5
Deaggregation_data(5:8,4) = (G3_RT)  /  n_repairable;                      % Path B: Repair time for RS3
Deaggregation_data(9:12,4) = (G6_RT)  /  n_repairable;                     % Path C: Repair time for RS6
Deaggregation_data(13:16,4) = (G7_RT)  /  n_repairable;                    % Path D: Repair time for RS7
xlswrite('Downtime_deaggregation.xlsx',Deaggregation_data,1)

%{
% Plot the vulnerability function
% Calculate the mean IF for irreparable realizations
IF_collapse = 0;
IF_demolition = 0;
for i=1:n_real
    if Collapse(1,i) == 1
        IF_collapse = IF_collapse + IF_reconstruction(1,i);
    end
    if Residual_Drift(1,i) == 1
        IF_demolition = IF_demolition + IF_reconstruction(1,i);
    end
end
MeanIF_collapse = IF_collapse/n_real;
MeanIF_demolition = IF_demolition/n_real;

% Vulnerability function is plotted considering repairable and irreparable
% realizations
MeanDowntime_Vulnerability(1,1) = MeanIF_collapse + 14*n_Floors*n_collapse/n_real;           % Collapse contribution
MeanDowntime_Vulnerability(2,1) = MeanIF_demolition + 14*n_Floors*n_RDdemolition/n_real;     % Residual drift contribution
MeanDowntime_Vulnerability(3,1) = max(sum(Deaggregation_data(1:16,:),2))  *  n_repairable / n_real;                    % Reparable contribution
MeanDowntime_Vulnerability(4,1) = sum(MeanDowntime_Vulnerability(1:3,1));                         % Total downtime

MeanBRT = 63.6*7 + 14*n_Floors;             % 63.6*7 is the median building demolition time in days 
Downtime_Ratio = min(MeanDowntime_Vulnerability ./ MeanBRT,1);

xlswrite('VulnerabilityFunction.xlsx',Downtime_Ratio,1)
%}

Downtime_deaggregation(1);

%% Plot the median stepping functions and the median downtimes for RC2, RC3 and RC4

%Median_downtime(n_Floors,n_Floors_AG);

%% Delete files
%delete(append(Folder_Path,'DT_Prct10_stepfunc_RC4.xlsx'))
%delete(append(Folder_Path,'DT_Medianstepfunc_RC4.xlsx'))
%delete(append(Folder_Path,'DT_Prct90_stepfunc_RC4.xlsx'))
%delete(append(Folder_Path,'DT_stepfunc_RC4.xlsx'))
% delete(append(Folder_Path,'DT_Prct10_stepfunc_RC3.xlsx'))
%delete(append(Folder_Path,'DT_Medianstepfunc_RC3.xlsx'))
% delete(append(Folder_Path,'DT_Prct90_stepfunc_RC3.xlsx'))
% delete(append(Folder_Path,'DT_stepfunc_RC3.xlsx'))
delete(append(Folder_Path,'DT_Prct10_stepfunc_RC2.xlsx'))
delete(append(Folder_Path,'DT_Medianstepfunc_RC2.xlsx'))
delete(append(Folder_Path,'DT_Prct90_stepfunc_RC2.xlsx'))
delete(append(Folder_Path,'DT_stepfunc_RC2.xlsx'))
delete(append(Folder_Path,'Reduced_RepairTime_RC2.xlsx'))
delete(append(Folder_Path,'Worker_Allocation_RC2.xlsx'))
delete(append(Folder_Path,'IF_CM.xlsx'))
delete(append(Folder_Path,'Utility_Downtime.xlsx'))
delete(append(Folder_Path,'AllReal_ImpedingFactors.xlsx'))
delete(append(Folder_Path,'RS_Number_DC_RC4.xlsx'))
delete(append(Folder_Path,'RS_Number_DC_RC3.xlsx'))
delete(append(Folder_Path,'RS_Number_DC_RC2.xlsx'))
delete(append(Folder_Path,'RS_Repair_Times_RC4.xlsx'))
delete(append(Folder_Path,'RS_Repair_Times_RC3.xlsx'))
delete(append(Folder_Path,'RS_Repair_Times_RC2.xlsx'))
delete(append(Folder_Path,'Repair_Times_RC2.xlsx'))
delete(append(Folder_Path,'AllReal_CollapseCases.xlsx'))
delete(append(Folder_Path,'RS_Story_Identifier.xlsx'))
delete(append(Folder_Path,'RS3_Stability.xlsx'))
delete(append(Folder_Path,'AllReal_RDcases.xlsx'))
delete(append(Folder_Path,'AllReal_TotalLoss.xlsx'))
delete(append(Folder_Path,'Number_DC_RC5.xlsx'))
delete(append(Folder_Path,'Number_DC_RC4.xlsx'))
delete(append(Folder_Path,'Number_DC_RC3.xlsx'))
delete(append(Folder_Path,'Number_DC_RC2.xlsx'))
writetable(Probabilities_LimitStates,append(Folder_Path,'Probabilities_LimitStates.xlsx'),'Sheet',1,'WriteRowNames',true)
writetable(Downtime_ConfidenceLevels_RC2,append(Folder_Path,'Downtime_ConfidenceLevels.xlsx'),'Sheet',1,'WriteRowNames',true)


end


%% INTRODUCTION 
%{
This is a MATLAB package to evaluate earthquake induced downtime and model recovery of buildings. This tool implements the framework presented in: 

Molina Hutt, C., Vahanvaty, T. and Kourehpaz, P. (2021). “An analytical framework to assess earthquake induced downtime and model recovery of buildings.” *Earthquake Spectra*, Accepted.

This tool is partially compatible with SimCenter’s tool for loss assessment, i.e., pelicun (https://github.com/NHERI-SimCenter/pelicun) 
%} 

%% NOTES:
% The framework is also coded in Python and provided as an open-source application called TREADS
%(Tool for Recovery Estimation And Downtime Simulation) at the following Github repository
%(https://github.com/carlosmolinahutt/treads). TREADS is also available at the Python PackageIndex (PyPI) and can be easily installed using pip. 
% The python framework is fully compatible with PELICUN. 
% Due to advantages of coding in python, the code is easier to read and the outputs may be easier to comprehend.  
% Further updates to this framework will be made only in TREADS (python version) and hence it is recommended to use the python code for downtime estimation.

%% README - INPUT GENERATION

% STEP 1:
% Create the Repair_Class_Table.xlsx (RCT matrix) with the following info for each PG: 
%   Col 1. Fragility identifier
%   Col 2. Row Identifier (Assign 1 to the first component, 2 to the second component and so on)
%   Col.3 to 6. RC=f(DS) Recovery state hindered by each DS
%   Col 7. Repair Sequence
%   Col 8. Qty of one fragility in FEMA P-58
%   Col 9. Qty of fragility in FEMA P-58
%   Col 10. Number of workers required for repair per damaged quantity
%   Col 11. Damage threshold that if exceeded across ALL floor hinders the shelter-in-place recovery state, i.e., 
%(The FEMA P-58 UNSAFE PLACARD Median damage limit for damage state corresponding to hindering shelter-in-place recovery state)
%   Col 12. Damage threshold that if exceeded across ALL floor hinders the stability recovery state, i.e.,  
%(The FEMA P-58 UNSAFE PLACARD Median damage limit for damage state corresponding to hindering stability recovery state)
%   Col 13. Damage threshold that if exceeded on any ONE floor hinders the shelter-in-place recovery state 
%   Col 14. Damage threshold that if exceeded on any ONE floor hinders the stability recovery state 



% STEP 2:
% Prepare the Floor&Direction_Identifier.xlsx database:
% 1st row: All fragility identifiers (copy from 1st row of 'DV_rec_time.csv')
% 2nd row: PG identifier from Pelicun (copy from 2nd row of 'DV_rec_time.csv') 
% 3rd row: Story identifier 
% 4th row: Direction identifier (1 & 2 for bidirectional components and 0 for unidirectional components)   
% 5th row: Group identifier for disaggregation 
%           Structural components are divided into groups 1a and 1b
%           Interior components are divided into 'Hot & cold water piping'...
%               'Wall partitions & finishing','Suspended ceiling','HVAC','Fire
%               sprinkler system' and identified as 2a,2b,2c,2d,2e respectively
%           Exterior, Mechanical equipment, Elevator and Staircase repairs
%               are identified only based on their repair sequence i.e. simply
%               a numerical number of their repair sequence (eg 3 for exterior
%               repairs)

 
% STEP 3:
% Create the new file 'Worker_Limitations.xlsx' with the worker
% limitations for each Repair sequence 
% Column 1: Repair Seqeunces identifier
% Column 2: Number of workers permitted per damaged components for each
%           repair seqeunce(As per Table 2: Improving estimates of Earthquake-Induced Downtime in Individual Buildings Using The REDi Methodology (2018))
% Column 3: Number of workers permitted for each repair sequence across all floors of the building (Depending on the number of floors in the building) 
%           (As per Table 1: Improving estimates of Earthquake-Induced Downtime in Individual Buildings Using The REDi Methodology (2018))
% Note: The user will also need to mention the floor area per floor of the building (in sq.ft) in the "Worker_Allocation.m" script

